"""BACnet enums."""

__all__ = ["ObjectType", "PropertyIdentifier"]

from enum import Enum


class ObjectType(Enum):
    """ObjectType enumerator."""

    analogInput = 0
    analogOutput = 1
    analogValue = 2
    binaryInput = 3
    binaryOutput = 4
    binaryValue = 5
    calendar = 6
    command = 7
    device = 8
    eventEnrollment = 9
    file = 10
    group = 11
    loop = 12
    multiStateInput = 13
    multiStateOutput = 14
    notificationClass = 15
    program = 16
    schedule = 17
    averaging = 18
    multiStateValue = 19
    trendLog = 20
    lifeSafetyPoint = 21
    lifeSafetyZone = 22
    accumulator = 23
    pulseConverter = 24
    eventLog = 25
    globalGroup = 26
    trendLogMultiple = 27
    loadControl = 28
    structuredView = 29
    accessDoor = 30
    timer = 31
    accessCredential = 32
    accessPoint = 33
    accessRights = 34
    accessUser = 35
    accessZone = 36
    credentialDataInput = 37
    networkSecurity = 38
    bitstringValue = 39
    characterstringValue = 40
    datePatternValue = 41
    dateValue = 42
    datetimePatternValue = 43
    datetimeValue = 44
    integerValue = 45
    largeAnalogValue = 46
    octetstringValue = 47
    positiveIntegerValue = 48
    timePatternValue = 49
    timeValue = 50
    notificationForwarder = 51
    alertEnrollment = 52
    channel = 53
    lightingOutput = 54
    binaryLightingOutput = 55
    networkPort = 56
    elevatorGroup = 57
    escalator = 58
    lift = 59


class PropertyIdentifier(Enum):
    """PropertyIdentifier enumerator."""

    ackedTransitions = 0
    ackRequired = 1
    action = 2
    actionText = 3
    activeText = 4
    activeVtSessions = 5
    alarmValue = 6
    alarmValues = 7
    all = 8
    allWritesSuccessful = 9
    apduSegmentTimeout = 10
    apduTimeout = 11
    applicationSoftwareVersion = 12
    archive = 13
    bias = 14
    changeOfStateCount = 15
    changeOfStateTime = 16
    notificationClass = 17
    controlledVariableReference = 19
    controlledVariableUnits = 20
    controlledVariableValue = 21
    covIncrement = 22
    dateList = 23
    daylightSavingsStatus = 24
    deadband = 25
    derivativeConstant = 26
    derivativeConstantUnits = 27
    description = 28
    descriptionOfHalt = 29
    deviceAddressBinding = 30
    deviceType = 31
    effectivePeriod = 32
    elapsedActiveTime = 33
    errorLimit = 34
    eventEnable = 35
    eventState = 36
    eventType = 37
    exceptionSchedule = 38
    faultValues = 39
    feedbackValue = 40
    fileAccessMethod = 41
    fileSize = 42
    fileType = 43
    firmwareRevision = 44
    highLimit = 45
    inactiveText = 46
    inProcess = 47
    instanceOf = 48
    integralConstant = 49
    integralConstantUnits = 50
    limitEnable = 52
    listOfGroupMembers = 53
    listOfObjectPropertyReferences = 54
    localDate = 56
    localTime = 57
    location = 58
    lowLimit = 59
    manipulatedVariableReference = 60
    maximumOutput = 61
    maxApduLengthAccepted = 62
    maxInfoFrames = 63
    maxMaster = 64
    maxPresValue = 65
    minimumOffTime = 66
    minimumOnTime = 67
    minimumOutput = 68
    minPresValue = 69
    modelName = 70
    modificationDate = 71
    notifyType = 72
    numberOfApduRetries = 73
    numberOfStates = 74
    objectIdentifier = 75
    objectList = 76
    objectName = 77
    objectPropertyReference = 78
    objectType = 79
    optional = 80
    outOfService = 81
    outputUnits = 82
    eventParameters = 83
    polarity = 84
    presentValue = 85
    priority = 86
    priorityArray = 87
    priorityForWriting = 88
    processIdentifier = 89
    programChange = 90
    programLocation = 91
    programState = 92
    proportionalConstant = 93
    proportionalConstantUnits = 94
    protocolObjectTypesSupported = 96
    protocolServicesSupported = 97
    protocolVersion = 98
    readOnly = 99
    reasonForHalt = 100
    recipientList = 102
    reliability = 103
    relinquishDefault = 104
    required = 105
    resolution = 106
    segmentationSupported = 107
    setpoint = 108
    setpointReference = 109
    stateText = 110
    statusFlags = 111
    systemStatus = 112
    timeDelay = 113
    timeOfActiveTimeReset = 114
    timeOfStateCountReset = 115
    timeSynchronizationRecipients = 116
    units = 117
    updateInterval = 118
    utcOffset = 119
    vendorIdentifier = 120
    vendorName = 121
    vtClassesSupported = 122
    weeklySchedule = 123
    attemptedSamples = 124
    averageValue = 125
    bufferSize = 126
    clientCovIncrement = 127
    covResubscriptionInterval = 128
    eventTimeStamps = 130
    logBuffer = 131
    logDeviceObjectProperty = 132
    enable = 133
    logInterval = 134
    maximumValue = 135
    minimumValue = 136
    notificationThreshold = 137
    protocolRevision = 139
    recordsSinceNotification = 140
    recordCount = 141
    startTime = 142
    stopTime = 143
    stopWhenFull = 144
    totalRecordCount = 145
    validSamples = 146
    windowInterval = 147
    windowSamples = 148
    maximumValueTimestamp = 149
    minimumValueTimestamp = 150
    varianceValue = 151
    activeCovSubscriptions = 152
    backupFailureTimeout = 153
    configurationFiles = 154
    databaseRevision = 155
    directReading = 156
    lastRestoreTime = 157
    maintenanceRequired = 158
    memberOf = 159
    mode = 160
    operationExpected = 161
    setting = 162
    silenced = 163
    trackingValue = 164
    zoneMembers = 165
    lifeSafetyAlarmValues = 166
    maxSegmentsAccepted = 167
    profileName = 168
    autoSlaveDiscovery = 169
    manualSlaveAddressBinding = 170
    slaveAddressBinding = 171
    slaveProxyEnable = 172
    lastNotifyRecord = 173
    scheduleDefault = 174
    acceptedModes = 175
    adjustValue = 176
    count = 177
    countBeforeChange = 178
    countChangeTime = 179
    covPeriod = 180
    inputReference = 181
    limitMonitoringInterval = 182
    loggingObject = 183
    loggingRecord = 184
    prescale = 185
    pulseRate = 186
    scale = 187
    scaleFactor = 188
    updateTime = 189
    valueBeforeChange = 190
    valueSet = 191
    valueChangeTime = 192
    alignIntervals = 193
    intervalOffset = 195
    lastRestartReason = 196
    loggingType = 197
    restartNotificationRecipients = 202
    timeOfDeviceRestart = 203
    timeSynchronizationInterval = 204
    trigger = 205
    utcTimeSynchronizationRecipients = 206
    nodeSubtype = 207
    nodeType = 208
    structuredObjectList = 209
    subordinateAnnotations = 210
    subordinateList = 211
    actualShedLevel = 212
    dutyWindow = 213
    expectedShedLevel = 214
    fullDutyBaseline = 215
    requestedShedLevel = 218
    shedDuration = 219
    shedLevelDescriptions = 220
    shedLevels = 221
    stateDescription = 222
    doorAlarmState = 226
    doorExtendedPulseTime = 227
    doorMembers = 228
    doorOpenTooLongTime = 229
    doorPulseTime = 230
    doorStatus = 231
    doorUnlockDelayTime = 232
    lockStatus = 233
    maskedAlarmValues = 234
    securedStatus = 235
    absenteeLimit = 244
    accessAlarmEvents = 245
    accessDoors = 246
    accessEvent = 247
    accessEventAuthenticationFactor = 248
    accessEventCredential = 249
    accessEventTime = 250
    accessTransactionEvents = 251
    accompaniment = 252
    accompanimentTime = 253
    activationTime = 254
    activeAuthenticationPolicy = 255
    assignedAccessRights = 256
    authenticationFactors = 257
    authenticationPolicyList = 258
    authenticationPolicyNames = 259
    authenticationStatus = 260
    authorizationMode = 261
    belongsTo = 262
    credentialDisable = 263
    credentialStatus = 264
    credentials = 265
    credentialsInZone = 266
    daysRemaining = 267
    entryPoints = 268
    exitPoints = 269
    expirationTime = 270
    extendedTimeEnable = 271
    failedAttemptEvents = 272
    failedAttempts = 273
    failedAttemptsTime = 274
    lastAccessEvent = 275
    lastAccessPoint = 276
    lastCredentialAdded = 277
    lastCredentialAddedTime = 278
    lastCredentialRemoved = 279
    lastCredentialRemovedTime = 280
    lastUseTime = 281
    lockout = 282
    lockoutRelinquishTime = 283
    maxFailedAttempts = 285
    members = 286
    musterPoint = 287
    negativeAccessRules = 288
    numberOfAuthenticationPolicies = 289
    occupancyCount = 290
    occupancyCountAdjust = 291
    occupancyCountEnable = 292
    occupancyLowerLimit = 294
    occupancyLowerLimitEnforced = 295
    occupancyState = 296
    occupancyUpperLimit = 297
    occupancyUpperLimitEnforced = 298
    passbackMode = 300
    passbackTimeout = 301
    positiveAccessRules = 302
    reasonForDisable = 303
    supportedFormats = 304
    supportedFormatClasses = 305
    threatAuthority = 306
    threatLevel = 307
    traceFlag = 308
    transactionNotificationClass = 309
    userExternalIdentifier = 310
    userInformationReference = 311
    userName = 317
    userType = 318
    usesRemaining = 319
    zoneFrom = 320
    zoneTo = 321
    accessEventTag = 322
    globalIdentifier = 323
    verificationTime = 326
    baseDeviceSecurityPolicy = 327
    distributionKeyRevision = 328
    doNotHide = 329
    keySets = 330
    lastKeyServer = 331
    networkAccessSecurityPolicies = 332
    packetReorderTime = 333
    securityPduTimeout = 334
    securityTimeWindow = 335
    supportedSecurityAlgorithms = 336
    updateKeySetTimeout = 337
    backupAndRestoreState = 338
    backupPreparationTime = 339
    restoreCompletionTime = 340
    restorePreparationTime = 341
    bitMask = 342
    bitText = 343
    isUtc = 344
    groupMembers = 345
    groupMemberNames = 346
    memberStatusFlags = 347
    requestedUpdateInterval = 348
    covuPeriod = 349
    covuRecipients = 350
    eventMessageTexts = 351
    eventMessageTextsConfig = 352
    eventDetectionEnable = 353
    eventAlgorithmInhibit = 354
    eventAlgorithmInhibitRef = 355
    timeDelayNormal = 356
    reliabilityEvaluationInhibit = 357
    faultParameters = 358
    faultType = 359
    localForwardingOnly = 360
    processIdentifierFilter = 361
    subscribedRecipients = 362
    portFilter = 363
    authorizationExemptions = 364
    allowGroupDelayInhibit = 365
    channelNumber = 366
    controlGroups = 367
    executionDelay = 368
    lastPriority = 369
    writeStatus = 370
    propertyList = 371
    serialNumber = 372
    blinkWarnEnable = 373
    defaultFadeTime = 374
    defaultRampRate = 375
    defaultStepIncrement = 376
    egressTime = 377
    inProgress = 378
    instantaneousPower = 379
    lightingCommand = 380
    lightingCommandDefaultPriority = 381
    maxActualValue = 382
    minActualValue = 383
    power = 384
    transition = 385
    egressActive = 386
    interfaceValue = 387
    faultHighLimit = 388
    faultLowLimit = 389
    lowDiffLimit = 390
    strikeCount = 391
    timeOfStrikeCountReset = 392
    defaultTimeout = 393
    initialTimeout = 394
    lastStateChange = 395
    stateChangeValues = 396
    timerRunning = 397
    timerState = 398
    apduLength = 399
    ipAddress = 400
    ipDefaultGateway = 401
    ipDhcpEnable = 402
    ipDhcpLeaseTime = 403
    ipDhcpLeaseTimeRemaining = 404
    ipDhcpServer = 405
    ipDnsServer = 406
    bacnetIpGlobalAddress = 407
    bacnetIpMode = 408
    bacnetIpMulticastAddress = 409
    bacnetIpNatTraversal = 410
    ipSubnetMask = 411
    bacnetIpUdpPort = 412
    bbmdAcceptFdRegistrations = 413
    bbmdBroadcastDistributionTable = 414
    bbmdForeignDeviceTable = 415
    changesPending = 416
    command = 417
    fdBbmdAddress = 418
    fdSubscriptionLifetime = 419
    linkSpeed = 420
    linkSpeeds = 421
    linkSpeedAutonegotiate = 422
    macAddress = 423
    networkInterfaceName = 424
    networkNumber = 425
    networkNumberQuality = 426
    networkType = 427
    routingTable = 428
    virtualMacAddressTable = 429
    commandTimeArray = 430
    currentCommandPriority = 431
    lastCommandTime = 432
    valueSource = 433
    valueSourceArray = 434
    bacnetIpv6Mode = 435
    ipv6Address = 436
    ipv6PrefixLength = 437
    bacnetIpv6UdpPort = 438
    ipv6DefaultGateway = 439
    bacnetIpv6MulticastAddress = 440
    ipv6DnsServer = 441
    ipv6AutoAddressingEnable = 442
    ipv6DhcpLeaseTime = 443
    ipv6DhcpLeaseTimeRemaining = 444
    ipv6DhcpServer = 445
    ipv6ZoneIndex = 446
    assignedLandingCalls = 447
    carAssignedDirection = 448
    carDoorCommand = 449
    carDoorStatus = 450
    carDoorText = 451
    carDoorZone = 452
    carDriveStatus = 453
    carLoad = 454
    carLoadUnits = 455
    carMode = 456
    carMovingDirection = 457
    carPosition = 458
    elevatorGroup = 459
    energyMeter = 460
    energyMeterRef = 461
    escalatorMode = 462
    faultSignals = 463
    floorText = 464
    groupId = 465
    groupMode = 467
    higherDeck = 468
    installationId = 469
    landingCalls = 470
    landingCallControl = 471
    landingDoorStatus = 472
    lowerDeck = 473
    machineRoomId = 474
    makingCarCall = 475
    nextStoppingFloor = 476
    operationDirection = 477
    passengerAlarm = 478
    powerMode = 479
    registeredCarCall = 480
    activeCovMultipleSubscriptions = 481
    protocolLevel = 482
    referencePort = 483
    deployedProfileLocation = 484
    profileLocation = 485
    tags = 486
    subordinateNodeTypes = 487
    subordinateTags = 488
    subordinateRelationships = 489
    defaultSubordinateRelationship = 490
    represents = 491



from __future__ import print_function

from typing import Any, Iterator, List, Optional

from java.io import Writer
from java.lang import Number, Object, String


class JSONArray(Object):
    def __init__(self, arg=None):
        # type: (Any) -> None
        print(arg)

    def get(self, index):
        # type: (int) -> Object
        pass

    def getBoolean(self, index):
        # type: (int) -> bool
        pass

    def getDouble(self, index):
        # type: (int) -> float
        pass

    def getInt(self, index):
        # type: (int) -> int
        pass

    def getJSONArray(self, index):
        # type: (int) -> JSONArray
        pass

    def getJSONObject(self, index):
        # type: (int) -> JSONObject
        pass

    def getLong(self, index):
        # type: (int) -> long
        pass

    def getString(self, index):
        # type: (int) -> String
        pass

    def isNull(self, index):
        # type: (int) -> bool
        pass

    def join(self, separator):
        # type: (String) -> String
        pass

    def length(self):
        # type: () -> int
        pass

    def opt(self, index):
        # type: (int) -> Object
        pass

    def optBoolean(self, index, defaultValue=None):
        # type: (int, Optional[bool]) -> bool
        pass

    def optDouble(self, index, defaultValue=None):
        # type: (int, Optional[float]) -> float
        pass

    def optInt(self, index, defaultValue=None):
        # type: (int, Optional[int]) -> int
        pass

    def optJSONArray(self, index):
        # type: (int) -> JSONArray
        pass

    def optJSONObject(self, index):
        # type: (int) -> JSONObject
        pass

    def optLong(self, index, defaultValue=None):
        # type: (int, Optional[long]) -> long
        pass

    def optString(self, index, defaultValue=None):
        # type: (int, Optional[String]) -> String
        pass

    def put(self, index, value):
        # type: (int, Any) -> JSONArray
        pass

    def remove(self, index):
        # type: (int) -> Object
        pass

    def toJSONObject(self, names):
        # type: (JSONArray) -> JSONObject
        pass

    def toString(self, indentFactor=None):
        # type: (Optional[int]) -> String
        pass

    def write(self, writer):
        # type: (Writer) -> Writer
        pass


class JSONObject(Object):
    NULL = None  # type: Object

    def __init__(self, *args):
        # type: (Any) -> None
        pass

    def accumulate(self, key, value):
        # type: (String, Object) -> JSONObject
        pass

    def append(self, key, value):
        # type: (String, Object) -> JSONObject
        pass

    @staticmethod
    def doubleToString(d):
        # type: (float) -> String
        pass

    def get(self, key):
        # type: (String) -> Object
        pass

    def getBoolean(self, key):
        # type: (String) -> bool
        pass

    def getDouble(self, key):
        # type: (String) -> float
        pass

    def getInt(self, key):
        # type: (String) -> int
        pass

    def getJSONArray(self, key):
        # type: (String) -> JSONArray
        pass

    def getJSONObject(self, key):
        # type: (String) -> JSONObject
        pass

    def getLong(self, key):
        # type: (String) -> long
        pass

    @staticmethod
    def getNames(arg):
        # type: (JSONObject) -> List[String]
        pass

    def getString(self, key):
        # type: (String) -> String
        pass

    def has(self, key):
        # type: (String) -> bool
        pass

    def increment(self, key):
        # type: (String) -> JSONObject
        pass

    def isNull(self, key):
        # type: (String) -> bool
        pass

    def keys(self):
        # type: () -> Iterator[String]
        pass

    def length(self):
        # type: () -> int
        pass

    def names(self):
        # type: () -> JSONArray
        pass

    @staticmethod
    def numberToString(n):
        # type: (Number) -> String
        pass

    def opt(self, key):
        # type: (String) -> Object
        pass

    def optBoolean(self, key, defaultValue=None):
        # type: (String, Optional[bool]) -> bool
        pass

    def optDouble(self, key, defaultValue=None):
        # type: (String, Optional[float]) -> float
        pass

    def optInt(self, key, defaultValue=None):
        # type: (String, Optional[int]) -> int
        pass

    def optJSONArray(self, key):
        # type: (String) -> JSONArray
        pass

    def optJSONObject(self, key):
        # type: (String) -> JSONObject
        pass

    def optLong(self, key, defaultValue=None):
        # type: (String, Optional[long]) -> long
        pass

    def optString(self, key, defaultValue=None):
        # type: (String, Optional[String]) -> String
        pass

    def put(self, key, value):
        # type: (String, Any) -> JSONObject
        pass

    def putOnce(self, key, value):
        # type: (String, Object) -> JSONObject
        pass

    def putOpt(self, key, value):
        # type: (String, Object) -> JSONObject
        pass

    @staticmethod
    def quote(string):
        # type: (String) -> String
        pass

    def remove(self, key):
        # type: (String) -> Object
        pass

    def sortedKeys(self):
        # type: () -> Iterator[String]
        pass

    @staticmethod
    def stringToValue(s):
        # type: (String) -> Object
        pass

    def toJSONArray(self, names):
        # type: (JSONArray) -> JSONArray
        pass

    def toString(self, indentFactor=None):
        # type: (Optional[int]) -> String
        pass

    @staticmethod
    def valueToString(value):
        # type: (Object) -> String
        pass

    @staticmethod
    def wrap(obj):
        # type: (Object) -> Object
        pass

    def write(self, writer):
        # type: (Writer) -> Writer
        pass



from typing import Any, Optional, Union

from java.lang import Appendable, CharSequence, Object

__all__ = [
    "Buffer",
    "ByteBuffer",
    "ByteOrder",
    "CharBuffer",
    "DoubleBuffer",
    "FloatBuffer",
    "IntBuffer",
    "LongBuffer",
    "MappedByteBuffer",
    "ShortBuffer",
]


class Buffer(Object):
    def array(self):
        # type: () -> Object
        raise NotImplementedError

    def arrayOffset(self):
        # type: () -> int
        raise NotImplementedError

    def capacity(self):
        # type: () -> int
        pass

    def clear(self):
        # type: () -> Buffer
        pass

    def duplicate(self):
        # type: () -> Buffer
        raise NotImplementedError

    def flip(self):
        # type: () -> Buffer
        pass

    def hasArray(self):
        # type: () -> bool
        raise NotImplementedError

    def hasRemaining(self):
        # type: () -> bool
        pass

    def isDirect(self):
        # type: () -> bool
        raise NotImplementedError

    def isReadOnly(self):
        # type: () -> bool
        raise NotImplementedError

    def limit(self, nrewLimit=None):
        # type: (Optional[int]) -> Union[Buffer, int]
        pass

    def mark(self):
        # type: () -> Buffer
        pass

    def position(self, newPosition=None):
        # type: (Optional[int]) -> Union[Buffer, int]
        pass

    def remaining(self):
        # type: () -> int
        pass

    def reset(self):
        # type: () -> Buffer
        pass

    def rewind(self):
        # type: () -> Buffer
        pass

    def slice(self):
        # type: () -> Buffer
        raise NotImplementedError


class ByteBuffer(Buffer):
    def alignedSlice(self, unitSize):
        # type: (int) -> ByteBuffer
        pass

    def alignmentOffset(self, index, unitSize):
        # type: (int, int) -> int
        pass

    @staticmethod
    def allocate(capacity):
        # type: (int) -> ByteBuffer
        pass

    @staticmethod
    def allocateDirect(capacity):
        # type: (int) -> ByteBuffer
        pass

    def array(self):
        # type: () -> Object
        pass

    def arrayOffset(self):
        # type: () -> int
        pass

    def asCharBuffer(self):
        # type: () -> CharBuffer
        pass

    def asDoubleBuffer(self):
        # type: () -> DoubleBuffer
        pass

    def asFloatBuffer(self):
        # type: () -> FloatBuffer
        pass

    def asIntBuffer(self):
        # type: () -> IntBuffer
        pass

    def asLongBuffer(self):
        # type: () -> LongBuffer
        pass

    def asReadOnlyBuffer(self):
        # type: () -> ByteBuffer
        pass

    def asShortBuffer(self):
        # type: () -> ShortBuffer
        pass

    def compact(self):
        # type: () -> ByteBuffer
        pass

    def compareTo(self, that):
        # type: (ByteBuffer) -> int
        pass

    def duplicate(self):
        # type: () -> Buffer
        pass

    def get(self, *args):
        # type: (Any) -> ByteBuffer
        pass

    def getChar(self, index=0):
        # type: (int) -> str
        pass

    def getDouble(self, index=0):
        # type: (int) -> float
        pass

    def getFloat(self, index=0):
        # type: (int) -> float
        pass

    def getInt(self, index=0):
        # type: (int) -> int
        pass

    def getLong(self, index=0):
        # type: (int) -> long
        pass

    def getShort(self, index=0):
        # type: (int) -> int
        pass

    def hasArray(self):
        # type: () -> bool
        pass

    def isDirect(self):
        # type: () -> bool
        pass

    def isReadOnly(self):
        # type: () -> bool
        pass

    def mismatch(self, that):
        # type: (ByteBuffer) -> int
        pass

    def order(self, bo=None):
        # type: (Optional[ByteOrder]) -> ByteOrder
        pass

    def put(self, *args):
        # type: (Any) -> ByteBuffer
        pass

    def putChar(self, *args):
        # type: (Any) -> ByteBuffer
        pass

    def putDouble(self, *args):
        # type: (Any) -> ByteBuffer
        pass

    def putFloat(self, *args):
        # type: (Any) -> ByteBuffer
        pass

    def putInt(self, *args):
        # type: (Any) -> ByteBuffer
        pass

    def putLong(self, *args):
        # type: (Any) -> ByteBuffer
        pass

    def putShort(self, *args):
        # type: (Any) -> ByteBuffer
        pass

    def slice(self):
        # type: () -> ByteBuffer
        pass

    @staticmethod
    def wrap(*arg):
        # type: (Any) -> ByteBuffer
        pass


class ByteOrder(Object):
    BIG_ENDIAN = None  # type: ByteOrder
    LITTLE_ENDIAN = None  # type: ByteOrder

    @staticmethod
    def nativeOrder():
        # type: () -> ByteOrder
        pass


class CharBuffer(Buffer, Appendable, CharSequence):
    @staticmethod
    def allocate(capacity):
        # type: (int) -> CharBuffer
        pass

    def append(self, c_csq, start=0, end=-1):
        # type: (Union[CharSequence, str], int, int) -> CharBuffer
        pass

    def array(self):
        # type: () -> Object
        pass

    def arrayOffset(self):
        # type: () -> int
        pass

    def charAt(self, index):
        # type: (int) -> str
        pass

    def compact(self):
        # type: () -> CharBuffer
        pass

    def compareTo(self, that):
        # type: (CharBuffer) -> int
        pass

    def duplicate(self):
        # type: () -> CharBuffer
        pass

    def get(self, *args):
        # type: (Any) -> CharBuffer
        pass

    def hasArray(self):
        # type: () -> bool
        pass

    def isDirect(self):
        # type: () -> bool
        pass

    def isReadOnly(self):
        # type: () -> bool
        pass

    def length(self):
        # type: () -> int
        pass

    def mismatch(self, that):
        # type: (CharBuffer) -> int
        pass

    def order(self):
        # type: () -> ByteOrder
        pass

    def put(self, *args):
        # type: (Any) -> CharBuffer
        pass

    def read(self, target):
        # type: (CharBuffer) -> int
        pass

    def slice(self):
        # type: () -> CharBuffer
        pass

    def subSequence(self, start, end):
        # type: (int, int) -> CharSequence
        pass

    @staticmethod
    def wrap(*args):
        # type: (Any) -> CharBuffer
        pass


class DoubleBuffer(Buffer):
    @staticmethod
    def allocate(capacity):
        # type: (int) -> DoubleBuffer
        pass

    def array(self):
        # type: () -> Object
        pass

    def arrayOffset(self):
        # type: () -> int
        pass

    def compact(self):
        # type: () -> DoubleBuffer
        pass

    def compareTo(self, that):
        # type: (DoubleBuffer) -> int
        pass

    def duplicate(self):
        # type: () -> DoubleBuffer
        pass

    def get(self, *args):
        # type: (Any) -> DoubleBuffer
        pass

    def hasArray(self):
        # type: () -> bool
        pass

    def isDirect(self):
        # type: () -> bool
        pass

    def isReadOnly(self):
        # type: () -> bool
        pass

    def mismatch(self, that):
        # type: (DoubleBuffer) -> int
        pass

    def order(self):
        # type: () -> ByteOrder
        pass

    def put(self, *args):
        # type: (Any) -> DoubleBuffer
        pass

    def slice(self):
        # type: () -> DoubleBuffer
        pass

    @staticmethod
    def wrap(*args):
        # type: (Any) -> DoubleBuffer
        pass


class FloatBuffer(Buffer):
    @staticmethod
    def allocate(capacity):
        # type: (int) -> FloatBuffer
        pass

    def array(self):
        # type: () -> Object
        pass

    def arrayOffset(self):
        # type: () -> int
        pass

    def compact(self):
        # type: () -> FloatBuffer
        pass

    def compareTo(self, that):
        # type: (FloatBuffer) -> int
        pass

    def duplicate(self):
        # type: () -> FloatBuffer
        pass

    def get(self, *args):
        # type: (Any) -> FloatBuffer
        pass

    def hasArray(self):
        # type: () -> bool
        pass

    def isDirect(self):
        # type: () -> bool
        pass

    def isReadOnly(self):
        # type: () -> bool
        pass

    def mismatch(self, that):
        # type: (FloatBuffer) -> int
        pass

    def order(self):
        # type: () -> ByteOrder
        pass

    def put(self, *args):
        # type: (Any) -> FloatBuffer
        pass

    def slice(self):
        # type: () -> FloatBuffer
        pass

    @staticmethod
    def wrap(*args):
        # type: (Any) -> FloatBuffer
        pass


class IntBuffer(Buffer):
    @staticmethod
    def allocate(capacity):
        # type: (int) -> IntBuffer
        pass

    def array(self):
        # type: () -> Object
        pass

    def arrayOffset(self):
        # type: () -> int
        pass

    def compact(self):
        # type: () -> IntBuffer
        pass

    def compareTo(self, that):
        # type: (IntBuffer) -> int
        pass

    def duplicate(self):
        # type: () -> IntBuffer
        pass

    def get(self, *args):
        # type: (Any) -> IntBuffer
        pass

    def hasArray(self):
        # type: () -> bool
        pass

    def isDirect(self):
        # type: () -> bool
        pass

    def isReadOnly(self):
        # type: () -> bool
        pass

    def mismatch(self, that):
        # type: (IntBuffer) -> int
        pass

    def order(self):
        # type: () -> ByteOrder
        pass

    def put(self, *args):
        # type: (Any) -> IntBuffer
        pass

    def slice(self):
        # type: () -> IntBuffer
        pass

    @staticmethod
    def wrap(*args):
        # type: (Any) -> IntBuffer
        pass


class LongBuffer(Buffer):
    @staticmethod
    def allocate(capacity):
        # type: (int) -> LongBuffer
        pass

    def array(self):
        # type: () -> Object
        pass

    def arrayOffset(self):
        # type: () -> int
        pass

    def compact(self):
        # type: () -> LongBuffer
        pass

    def compareTo(self, that):
        # type: (LongBuffer) -> int
        pass

    def duplicate(self):
        # type: () -> LongBuffer
        pass

    def get(self, *args):
        # type: (Any) -> LongBuffer
        pass

    def hasArray(self):
        # type: () -> bool
        pass

    def isDirect(self):
        # type: () -> bool
        pass

    def isReadOnly(self):
        # type: () -> bool
        pass

    def mismatch(self, that):
        # type: (LongBuffer) -> int
        pass

    def order(self):
        # type: () -> ByteOrder
        pass

    def put(self, *args):
        # type: (Any) -> LongBuffer
        pass

    def slice(self):
        # type: () -> LongBuffer
        pass

    @staticmethod
    def wrap(*args):
        # type: (Any) -> LongBuffer
        pass


class ShortBuffer(Buffer):
    @staticmethod
    def allocate(capacity):
        # type: (int) -> ShortBuffer
        pass

    def array(self):
        # type: () -> Object
        pass

    def arrayOffset(self):
        # type: () -> int
        pass

    def compact(self):
        # type: () -> ShortBuffer
        pass

    def compareTo(self, that):
        # type: (ShortBuffer) -> int
        pass

    def duplicate(self):
        # type: () -> ShortBuffer
        pass

    def get(self, *args):
        # type: (Any) -> ShortBuffer
        pass

    def hasArray(self):
        # type: () -> bool
        pass

    def isDirect(self):
        # type: () -> bool
        pass

    def isReadOnly(self):
        # type: () -> bool
        pass

    def mismatch(self, that):
        # type: (ShortBuffer) -> int
        pass

    def order(self):
        # type: () -> ByteOrder
        pass

    def put(self, *args):
        # type: (Any) -> ShortBuffer
        pass

    def slice(self):
        # type: () -> ShortBuffer
        pass

    @staticmethod
    def wrap(*args):
        # type: (Any) -> ShortBuffer
        pass


class MappedByteBuffer(ByteBuffer):
    def force(self):
        # type: () -> MappedByteBuffer
        pass

    def isLoaded(self):
        # type: () -> bool
        pass

    def load(self):
        # type: () -> MappedByteBuffer
        pass



from __future__ import print_function

__all__ = ["FPMIApp", "FPMIWindow", "VisionDesktop"]

from com.inductiveautomation.ignition.client.model import ClientContext
from java.lang import String
from javax.swing import JDesktopPane, JInternalFrame


class VisionDesktop(JDesktopPane):
    PRIMARY_DESKTOP_HANDLE = "Primary"

    def __init__(self, clientContext, handle=""):
        # type: (ClientContext, String) -> None
        super(VisionDesktop, self).__init__()
        print(clientContext, handle)

    def getAdapterContext(self):
        pass

    def getHandle(self):
        pass

    def getOpenedWindows(self):
        pass

    def getPath(self, window):
        pass

    def getScriptManager(self):
        pass

    def getWindow(self, arg):
        pass


class FPMIApp(VisionDesktop):
    TIMEZONE_CLIENT = "America/Tijuana"
    TIMEZONE_GATEWAY = "America/Tijuana"

    def getDefaultTimeZone(self):
        pass

    def getLastActivity(self):
        pass

    def shutdownDesigner(self):
        pass

    def startupDesigner(self, gatewayTimeZone):
        pass


class FPMIWindow(JInternalFrame):
    CACHE_ALWAYS = 2
    CACHE_AUTO = 0
    CACHE_NEVER = 1
    DOCK_EAST = 2
    DOCK_FLOAT = 0
    DOCK_NORTH = 2
    DOCK_SOUTH = 4
    DOCK_WEST = 3
    PARENT_WINDOW_NAME = "_parent"
    SHOW_ALWAYS = 0
    SHOW_NEVER = 1
    SHOW_MAXIMIZED = 2

    name = ""  # type: String
    _path = "Path/To/Window"

    def __init__(self, name):
        # type: (String) -> None
        super(FPMIWindow, self).__init__()
        self.name = name

    def getPath(self):
        return self._path

    def getRootContainer(self):
        pass



"""Alarm Functions.

The following functions give you access to view and interact with the
Alarm system in Ignition.
"""

from __future__ import print_function

__all__ = [
    "acknowledge",
    "cancel",
    "createRoster",
    "getRosters",
    "getShelvedPaths",
    "listPipelines",
    "queryJournal",
    "queryStatus",
    "shelve",
    "unshelve",
]

from typing import Any, Dict, List, Optional, Tuple, Union

from com.inductiveautomation.ignition.common.alarming.evaluation import ShelvedPath
from com.inductiveautomation.ignition.common.alarming.query import AlarmQueryResultImpl
from java.lang import String
from java.util import Date


def acknowledge(alarmIds, notes, username=None):
    # type: (List[String], String, Optional[String]) -> None
    """Acknowledges any number of alarms, specified by their event ids.

    The event id is generated for an alarm when it becomes active, and
    is used to identify a particular event from other events for the
    same source. The alarms will be acknowledged by the logged in user
    making the call. Additionally, acknowledgement notes may be included
    and will be stored along with the acknowledgement.

    Note:
         The `username` parameter is only used when called from a
         gateway scoped script. This parameter should be omitted from
         any client-based scripts.

    Args:
        alarmIds: List of alarm event ids (UUIDs) to acknowledge.
        notes: A string that will be used as the Ack Note on each
            acknowledged alarm event. If set to None, then an Ack Note
            note will not be assigned to the alarm event.
        username: The user that acknowledged the alarm.
    """
    print(alarmIds, notes, username)


def cancel(alarmIds):
    # type: (List[String]) -> None
    """Cancels any number of alarms, specified by their event ids.

    Event ids can be obtained from the system.alarm.queryStatus
    function. Canceling a pipeline will not impact the alarm that
    triggered the pipeline. The alarm will still be active, but will
    drop out of alarm pipelines.

    Args:
        alarmIds: List of alarm event ids (UUIDs) to cancel.
    """
    print(alarmIds)


def createRoster(name, description):
    # type: (String, String) -> None
    """This function creates a new roster.

    Users may be added to the roster through the Gateway or the Roster
    Management component.

    Args:
        name: The name for the new roster.
        description: A description for the new roster. Required, but can
            be blank.
    """
    print(name, description)


def getRosters():
    # type: () -> Dict[String, List[String]]
    """This function returns a mapping of roster names to a list of
    usernames contained in the roster.

    Returns:
        A dictionary that maps roster names to a List of usernames in
        the roster. The list of usernames may be empty if no users have
        been added to the roster.
    """
    return {}


def getShelvedPaths():
    # type: () -> List[ShelvedPath]
    """Returns a list of ShelvedPath objects, which each represent a
    shelved alarm.

    Returns:
        A list of ShelvedPath objects.
    """
    return [ShelvedPath()]


def listPipelines(projectName="alarm-pipelines"):
    # type: (String) -> List[String]
    """Will return a list of the available Alarm Notification Pipelines.

    Args:
        projectName: The project to check alarm pipelines for. If
            omitted, will look for a project named "alarm-pipelines".
            Optional.

    Returns:
        A list of pipeline names. The list may be empty if no pipelines
        exist. Unsaved name changes will not be reflected in the list.
    """
    print(projectName)
    return []


def queryJournal(
    startDate=None,  # type: Optional[Date]
    endDate=None,  # type: Optional[Date]
    journalName=None,  # type: Optional[String]
    priority=None,  # type: Optional[List[Union[String, int]]]
    state=None,  # type: Optional[List[Union[String, int]]]
    path=None,  # type: Optional[List[String]]
    source=None,  # type: Optional[List[String]]
    displaypath=None,  # type: Optional[List[String]]
    all_properties=None,  # type: Optional[List[Tuple[String, String, Any]]]
    any_properties=None,  # type: Optional[List[Tuple[String, String, Any]]]
    defined=None,  # type: Optional[List[String]]
    includeData=None,  # type: Optional[bool]
    includeSystem=None,  # type: Optional[bool]
    isSystem=None,  # type: Optional[bool]
):
    # type: (...) -> AlarmQueryResultImpl
    """Queries the specified journal for historical alarm events.

    The result is a list of alarm events, which can be queried for
    individual properties.

    Note:
        Each item in the resulting object is a separate alarm event: an
        alarm becoming active is one item, while the same alarm becoming
        acknowledged is a separate item. This differs from
        `system.alarm.queryStatus()` which groups each event into a
        single item.

    Args:
        startDate: The start of the time range to query. Defaults to 8
            hours previous to now if omitted. Time range is inclusive.
            Optional.
        endDate: The end of the time range to query. Defaults to "now"
            if omitted. Optional.
        journalName: The journal name to query. If only one journal
            exists on the Gateway, can be omitted. Optional.
        priority: A list of possible priorities to match. Priorities can
            be specified by name or number, with the values:
            Diagnostic(0), Low(1), Medium(2), High(3), Critical(4).
            Optional.
        state: A list of the event state types to match. Valid values
            can either be integers or strings, representing a number of
            states. Optional.
        path: A list of possible source paths to search at. The wildcard
            "*" may be used. Optional.
        source: A list of possible source paths to search at. The
            wildcard "*" may be used. Optional.
        displaypath: A list of display paths to search at. Display paths
            are separated by "/", and if a path ends in "/*", everything
            below that path will be searched as well.
        all_properties: A set of property conditions, all of which must
            be met for the condition to pass. This parameter is a list
            of tuples, in the form ("propName", "condition", value).
            Valid condition values: "=", "!=", "<", "<=", ">", ">=".
            String values can only be compared using "=" and "!="
            conditions. Optional.
        any_properties: A set of property conditions, any of which will
            cause the overall condition to pass. This parameter is a
            list of tuples, in the form ("propName", "condition",
            value). Valid condition values: "=", "!=", "<", "<=", ">",
            ">=". String values can only be compared using "=" and "!="
            conditions. Optional.
        defined: A list of string property names, all of which must be
            present on an event for it to pass. Optional.
        includeData: Whether or not event data should be included in the
            return. If True, returns Python dictionaries (or nulls) for
            Active Data, Clear Data, Ack Data, Runtime Data inside of
            the AlarmQueryResult object. Optional.
        includeSystem: Specifies whether system events are included in
            the return. Optional.
        isSystem: Specifies whether the returned event must or must not
            be a system event. Optional.

    Returns:
        The AlarmQueryResult object is functionally a list of AlarmEvent
        objects.
    """
    print(
        startDate,
        endDate,
        journalName,
        priority,
        state,
        path,
        source,
        displaypath,
        all_properties,
        any_properties,
        defined,
        includeData,
        includeSystem,
        isSystem,
    )
    return AlarmQueryResultImpl()


def queryStatus(
    priority=None,  # type: Optional[List[Union[String, int]]]
    state=None,  # type: Optional[List[Union[String, int]]]
    path=None,  # type: Optional[List[String]]
    source=None,  # type: Optional[List[String]]
    displaypath=None,  # type: Optional[List[String]]
    all_properties=None,  # type: Optional[List[Tuple[String, String, Any]]]
    any_properties=None,  # type: Optional[List[Tuple[String, String, Any]]]
    defined=None,  # type: Optional[List[String]]
    includeShelved=False,  # type: bool
):
    # type: (...) -> AlarmQueryResultImpl
    """Queries the current state of alarms.

    The result is a list of alarm events, which can be queried for
    individual properties. The results provided by this function
    represent the current state of alarms, in contrast to the historical
    alarm events retrieved by the `system.alarm.queryJournal` function.

    Note:
        Depending on the number of alarm events in the system, this
        function can be fairly intensive and take a while to finish
        executing. This can be problematic if the application is
        attempting to show the results on a component (such as using
        this function to retrieve a count of alarms). In these cases
        it's preferred to call this function in a gateway script of some
        sort (such as a timer script), and store the results in a tag.

    Args:
        priority: A list of possible priorities to match. Priorities can
            be specified by name or number, with the values:
            Diagnostic(0), Low(1), Medium(2), High(3), Critical(4).
            Optional.
        state: A list of states to allow. Optional.
        path: A list of possible source paths to search at. The
            wildcard "*" may be used. Works the same as the source
            argument, and either can be used. Optional.
        source: A list of possible source paths to search at. The
            wildcard "*" may be used. Works the same as the path
            argument, and either can be used. Optional.
        displaypath: A list of display paths to search at. Display paths
            are separated by "/", and if a path ends in "/*", everything
            below that path will be searched as well. Optional.
        all_properties: A set of property conditions, all of which must
            be met for the condition to pass. This parameter is a list
            of tuples, in the form ("propName", "condition", value).
            Valid condition values: "=", "!=", "<", "<=", ">", ">=".
            String values can only be compared using "=" and "!="
            conditions. Optional.
        any_properties: A set of property conditions, any of which will
            cause the overall the condition to pass. This parameter is a
            list of tuples, in the form ("propName", "condition",
            value). Valid condition values: "=", "!=", "<", "<=", ">",
            ">=". String values can only be compared using "=" and "!="
            conditions. Optional.
        defined: A list of string property names, all of which must be
            present on an event for it to pass. Optional.
        includeShelved: A flag indicating whether shelved events should
            be included in the results. Defaults to False. Optional.

    Returns:
        The AlarmQueryResult object is functionally a list of AlarmEvent
        objects with some additional helper methods.
    """
    print(
        priority,
        state,
        path,
        source,
        displaypath,
        all_properties,
        any_properties,
        defined,
        includeShelved,
    )
    return AlarmQueryResultImpl()


def shelve(path, timeoutSeconds=0, timeoutMinutes=15):
    # type: (List[String], int, int) -> None
    """This function shelves the specified alarms for the specified
    amount of time.

    The time can be specified in minutes (timeoutMinutes) or seconds
    (timeoutSeconds). If an alarm is already shelved, this will
    overwrite the remaining time. If no timeout is specified, will
    default to 15 minutes.

    Args:
        path: A list of possible source paths to search at. If a path
            ends in "/*", the results will include anything below that
            path.
        timeoutSeconds: The amount of time to shelve the matching alarms
            for, specified in seconds. Setting this to 0 will unshelve
            the alarms. Optional.
        timeoutMinutes: The amount of time to shelve the matching alarms
            for, specified in minutes. Setting this to 0 will unshelve
            the alarms. Optional.
    """
    print(path, timeoutSeconds, timeoutMinutes)


def unshelve(path):
    # type: (List[String]) -> None
    """Unshelves a list of alarms based on the source paths provided.

    Args:
        path: A list of possible source paths to search at. If a path
            ends in "/*", the results will include anything below that
            path.
    """
    print(path)



__all__ = [
    "AbstractJythonMap",
    "AbstractJythonSequence",
    "AbstractMutableJythonMap",
    "AbstractMutableJythonSequence",
    "JythonMap",
    "JythonSequence",
    "MutableJythonMap",
    "MutableJythonSequence",
]


class JythonMap(object):
    def __contains__(self, item):
        raise NotImplementedError

    def __finditem__(self, key):
        raise NotImplementedError

    def __iter__(self):
        raise NotImplementedError

    def __len__(self):
        raise NotImplementedError

    def get(self, *args):
        raise NotImplementedError

    def has_key(self, pyKey):
        raise NotImplementedError

    def items(self):
        pass

    def iteritems(self):
        raise NotImplementedError

    def iterkeys(self):
        raise NotImplementedError

    def itervalues(self):
        raise NotImplementedError

    def keys(self):
        raise NotImplementedError

    def values(self):
        raise NotImplementedError


class JythonSequence(object):
    def __contains__(self, item):
        raise NotImplementedError

    def __iter__(self):
        raise NotImplementedError

    def __len__(self):
        raise NotImplementedError

    def __mul__(self, other):
        pass

    def __repr__(self):
        pass

    def __rmul__(self, other):
        raise NotImplementedError

    def count(self, element):
        raise NotImplementedError

    def index(self, element):
        raise NotImplementedError


class MutableJythonMap(object):
    def __delitem__(self, key):
        raise NotImplementedError

    def __setitem__(self, key, value):
        raise NotImplementedError

    def clear(self):
        raise NotImplementedError

    def pop(self, *args):
        raise NotImplementedError

    def popitem(self):
        raise NotImplementedError

    def setdefault(self, *args):
        raise NotImplementedError

    def update(self, *args, **kwargs):
        raise NotImplementedError


class MutableJythonSequence(object):
    def __add__(self, other):
        raise NotImplementedError

    def __imul__(self, other):
        raise NotImplementedError

    def append(self, element):
        raise NotImplementedError

    def extend(self, sequence):
        raise NotImplementedError

    def insert(self, index, element):
        raise NotImplementedError

    def pop(self, *args):
        raise NotImplementedError

    def remove(self, element):
        raise NotImplementedError

    def sort(self, *args, **kwargs):
        raise NotImplementedError


class AbstractJythonMap(JythonMap):
    def __contains__(self, item):
        pass

    def __finditem__(self, key):
        pass

    def __iter__(self):
        pass

    def __len__(self):
        pass

    def get(self, *args):
        pass

    def has_key(self, pyKey):
        pass

    def items(self):
        pass

    def iteritems(self):
        pass

    def iterkeys(self):
        pass

    def itervalues(self):
        pass

    def keys(self):
        pass

    def values(self):
        pass


class AbstractJythonSequence(JythonSequence):
    def __contains__(self, item):
        pass

    def __iter__(self):
        pass

    def __len__(self):
        pass

    def __mul__(self, other):
        pass

    def __repr__(self):
        pass

    def __rmul__(self, other):
        pass

    def count(self, element):
        pass

    def index(self, element):
        pass


class AbstractMutableJythonMap(MutableJythonMap):
    def __delitem__(self, key):
        pass

    def __setitem__(self, key, value):
        pass

    def clear(self):
        pass

    def pop(self, *args):
        pass

    def popitem(self):
        pass

    def setdefault(self, *args):
        pass

    def update(self, *args, **kwargs):
        pass


class AbstractMutableJythonSequence(MutableJythonSequence):
    def __add__(self, other):
        pass

    def __imul__(self, other):
        pass

    def append(self, element):
        pass

    def extend(self, sequence):
        pass

    def insert(self, index, element):
        pass

    def pop(self, *args):
        pass

    def remove(self, element):
        pass

    def sort(self, *args, **kwargs):
        pass



__all__ = ["PyChartScope"]

from java.lang import Object


class PyChartScope(Object):
    """This class represents any "scope" in the SFC system, and is
    fundamentally just an observable dictionary.

    Despite its name, it is not limited to chart scope. This class
    notifies listeners when values are changed, and wraps any
    dictionaries assigned to it as PyChartScopes as well.
    """

    def __set__(self, instance, value):
        pass

    def __setattr__(self, key, value):
        pass

    def __setitem__(self, key, value):
        pass



"""Serial Functions.

The following functions give you access to read and write through serial
ports.
"""

from __future__ import print_function

__all__ = [
    "BIT_RATE_110",
    "BIT_RATE_115200",
    "BIT_RATE_1200",
    "BIT_RATE_150",
    "BIT_RATE_19200",
    "BIT_RATE_230400",
    "BIT_RATE_2400",
    "BIT_RATE_300",
    "BIT_RATE_38400",
    "BIT_RATE_460800",
    "BIT_RATE_4800",
    "BIT_RATE_57600",
    "BIT_RATE_600",
    "BIT_RATE_921600",
    "BIT_RATE_9600",
    "DATA_BITS_5",
    "DATA_BITS_6",
    "DATA_BITS_7",
    "DATA_BITS_8",
    "HANDSHAKE_CTS_DTR",
    "HANDSHAKE_CTS_RTS",
    "HANDSHAKE_DSR_DTR",
    "HANDSHAKE_HARD_IN",
    "HANDSHAKE_HARD_OUT",
    "HANDSHAKE_NONE",
    "HANDSHAKE_SOFT_IN",
    "HANDSHAKE_SOFT_OUT",
    "HANDSHAKE_SPLIT_MASK",
    "HANDSHAKE_XON_XOFF",
    "PARITY_EVEN",
    "PARITY_MARK",
    "PARITY_NONE",
    "PARITY_ODD",
    "PARITY_SPACE",
    "STOP_BITS_1",
    "STOP_BITS_2",
    "closeSerialPort",
    "configureSerialPort",
    "openSerialPort",
    "port",
    "readBytes",
    "readBytesAsString",
    "readLine",
    "readUntil",
    "sendBreak",
    "write",
    "writeBytes",
]

from typing import Any, List, Optional

from com.inductiveautomation.ignition.modules.serial.scripting import SerialScriptModule
from java.lang import String

PortManager = SerialScriptModule.PortManager
SerialConfigurator = SerialScriptModule.SerialConfigurator

# Bit rate constants.
BIT_RATE_110 = 110
BIT_RATE_150 = 150
BIT_RATE_300 = 300
BIT_RATE_600 = 600
BIT_RATE_1200 = 1200
BIT_RATE_2400 = 2400
BIT_RATE_4800 = 4800
BIT_RATE_9600 = 9600
BIT_RATE_19200 = 19200
BIT_RATE_38400 = 38400
BIT_RATE_57600 = 57600
BIT_RATE_115200 = 115200
BIT_RATE_230400 = 230400
BIT_RATE_460800 = 460800
BIT_RATE_921600 = 921600

# Data bits constants.
DATA_BITS_5 = 5
DATA_BITS_6 = 6
DATA_BITS_7 = 7
DATA_BITS_8 = 8

# Handshake constants.
HANDSHAKE_CTS_DTR = 4112
HANDSHAKE_CTS_RTS = 17
HANDSHAKE_DSR_DTR = 4352
HANDSHAKE_HARD_IN = 0
HANDSHAKE_HARD_OUT = 0
HANDSHAKE_NONE = 0
HANDSHAKE_SOFT_IN = 65536
HANDSHAKE_SOFT_OUT = 1048576
HANDSHAKE_SPLIT_MASK = 0
HANDSHAKE_XON_XOFF = 1114112

# Parity constants.
PARITY_EVEN = 2
PARITY_ODD = 1
PARITY_MARK = 3
PARITY_SPACE = 4
PARITY_NONE = 0

# Stop bits constants.
STOP_BITS_1 = 1
STOP_BITS_2 = 3


def closeSerialPort(port):
    # type: (String) -> None
    """Closes a previously opened serial port.

    Returns without doing anything if the named serial port is not
    currently open. Will throw an exception if the port is open and
    cannot be closed.

    Args:
        port: The name of the serial port, e.g., "COM1" or "dev/ttyS0".
    """
    print(port)


def configureSerialPort(
    port,  # type: String
    bitRate=None,  # type: Optional[int]
    dataBits=None,  # type: Optional[int]
    handshake=None,  # type: Optional[int]
    hardwareFlowControl=None,  # type: Optional[bool]
    parity=None,  # type: Optional[int]
    stopBits=None,  # type: Optional[int]
):
    # type: (...) -> SerialConfigurator
    """Configure a serial port for use in a later call.

    This only needs to be done once unless the configuration has changed
    after the initial call. All access to constants must be prefixed by
    "system.serial.".

    Args:
        port: The name of the serial port, e.g., "COM1" or "/dev/ttyS0".
            This parameter is required.
        bitRate: Configure the bit rate. Valid values are defined by the
            following constants: BIT_RATE_110, BIT_RATE_150,
            BIT_RATE_300, BIT_RATE_600, BIT_RATE_1200, BIT_RATE_2400,
            BIT_RATE_4800, BIT_RATE_9600, BIT_RATE_19200,
            BIT_RATE_38400, BIT_RATE_57600, BIT_RATE_115200,
            BIT_RATE_230400, BIT_RATE_460800, BIT_RATE_921600. Optional.
        dataBits: Configure the data bits. Valid values are defined by
            the following constants: DATA_BITS_5, DATA_BITS_6,
            DATA_BITS_7, DATA_BITS_8. Optional.
        handshake: Configure the handshake. Valid values are defined by
            the following constants: HANDSHAKE_CTS_DTR,
            HANDSHAKE_CTS_RTS, HANDSHAKE_DSR_DTR, HANDSHAKE_HARD_IN,
            HANDSHAKE_HARD_OUT, HANDSHAKE_NONE, HANDSHAKE_SOFT_IN,
            HANDSHAKE_SOFT_OUT, HANDSHAKE_SPLIT_MASK,
            HANDSHAKE_XON_XOFF. Optional.
        hardwareFlowControl: Configure hardware flow control. On or off.
        parity: Configure parity. Valid values are defined by the
            following constants: PARITY_EVEN, PARITY_ODD, PARITY_MARK,
            PARITY_SPACE, PARITY_NONE. Optional.
        stopBits: Configure stop bits. Valid values are defined by the
            following constants: STOP_BITS_1, STOP_BITS_2. Optional.

    Returns:
        A SerialConfigurator object with exposed functions that can be
        used to configure the serial port instead of, or in addition to,
        the arguments passed to configureSerialPort.
    """
    print(
        port,
        bitRate,
        dataBits,
        handshake,
        hardwareFlowControl,
        parity,
        stopBits,
    )
    return SerialConfigurator()


def openSerialPort(port):
    # type: (String) -> None
    """Opens a previously configured serial port for use.

    Will throw an exception if the serial port cannot be opened.

    Args:
        port: The name of the serial port, e.g., "COM1" or "dev/ttyS0".
    """
    print(port)


def port(
    port,  # type: String
    bitRate=None,  # type: Optional[int]
    dataBits=None,  # type: Optional[int]
    handshake=None,  # type: Optional[int]
    hardwareFlowControl=None,  # type: Optional[bool]
    parity=None,  # type: Optional[int]
    stopBits=None,  # type: Optional[int]
):
    # type: (...) -> PortManager
    """Returns a context manager wrapping a serial port, allowing the
    rest of the system to interact with that port.

    This function effectively combines the
    system.serial.configureSerialPort, system.serial.openSerialPort,
    and system.serial.closeSerialPort functions into a single call.

    Intended to be used with the Python 'with' statement. The object
    aliased in the 'with' statement has special access to all of the
    other system.serial functions, allowing for reads and writes.

    Closing the port happens automatically once the 'with' statement
    ends.

    Accepts the same arguments as configureSerialPort, and access to
    constants must be prefixed by "system.serial." As shown in the
    parameter descriptions.

    Args:
        port: The name of the serial port, e.g., "COM1" or "dev/ttyS0".
        bitRate: Configure the bit rate. Valid values are defined by the
            following constants: BIT_RATE_110, BIT_RATE_150,
            BIT_RATE_300, BIT_RATE_600, BIT_RATE_1200, BIT_RATE_2400,
            BIT_RATE_4800, BIT_RATE_9600, BIT_RATE_19200,
            BIT_RATE_38400, BIT_RATE_57600, BIT_RATE_115200,
            BIT_RATE_230400, BIT_RATE_460800, BIT_RATE_921600. Optional.
        dataBits: Configure the data bits. Valid values are defined by
            the following constants: DATA_BITS_5, DATA_BITS_6,
            DATA_BITS_7, DATA_BITS_8. Optional.
        handshake: Configure the handshake. Valid values are defined by
            the following constants: HANDSHAKE_CTS_DTR,
            HANDSHAKE_CTS_RTS, HANDSHAKE_DSR_DTR, HANDSHAKE_HARD_IN,
            HANDSHAKE_HARD_OUT, HANDSHAKE_NONE, HANDSHAKE_SOFT_IN,
            HANDSHAKE_SOFT_OUT, HANDSHAKE_SPLIT_MASK,
            HANDSHAKE_XON_XOFF. Optional.
        hardwareFlowControl: Configure hardware flow control. On or off.
            Optional.
        parity: Configure parity. Valid values are defined by the
            following constants: PARITY_EVEN, PARITY_ODD, PARITY_MARK,
            PARITY_SPACE, PARITY_NONE. Optional.
        stopBits: Configure stop bits. Valid values are defined by the
            following constants: STOP_BITS_1, STOP_BITS_2. Optional.

    Returns:
        A wrapper around the configured port, that can be entered by
        using a 'with' statement. The port will automatically close on
        exiting the 'with' statement scope.
    """
    print(
        port,
        bitRate,
        dataBits,
        handshake,
        hardwareFlowControl,
        parity,
        stopBits,
    )
    return PortManager()


def readBytes(port, numberOfBytes, timeout=5000):
    # type: (String, int, Optional[int]) -> List[Any]
    """Read numberOfBytes bytes from a serial port.

    Args:
        port: The previously configured serial port to use.
        numberOfBytes: The number of bytes to read.
        timeout: Maximum amount of time, in milliseconds, to block
            before returning. Default is 5,000. Optional.

    Returns:
        A list containing bytes read from the serial port.
    """
    print(port, numberOfBytes, timeout)
    return []


def readBytesAsString(
    port,  # type: String
    numberOfBytes,  # type: int
    timeout=5000,  # type: int
    encoding="utf-8",  # type: String
):
    # type: (...) -> String
    """Read numberOfBytes bytes from a serial port and convert them to a
    String.

    If a specific encoding is needed to match the source of the data,
    use system.serial.readBytes and use the desired encoding to decode
    the byte array returned.

    Args:
        port: The previously configured serial port to use.
        numberOfBytes: The number of bytes to read.
        timeout: Maximum amount of time, in milliseconds, to block
            before returning. Default is 5,000. Optional.
        encoding: Encoding to use when constructing the string. Defaults
            to the platform's default character set. Optional.

    Returns:
        A String created from the bytes read.
    """
    print(port, numberOfBytes, timeout, encoding)
    return ""


def readLine(
    port,  # type: String
    timeout=5000,  # type: int
    encoding="utf-8",  # type: String
    crlfRequired=False,  # type: bool
):
    # type: (...) -> String
    r"""Attempts to read a line from a serial port.

    A "line" is considered to be terminated by either a line feed
    ('\n'), a carriage return ('\r'), or a carriage return followed
    immediately by a line feed.

    The function will wait until the timeout period for a terminator. If
    the timeout is reached before the line is properly terminated, then
    the buffer will be dumped, possibly resulting in data loss.

    Args:
        port: The previously configured serial port to use.
        timeout: Maximum amount of time, in milliseconds, to block
            before returning. Default is 5000. Optional.
        encoding: The String encoding to use. Default is UTF8. Optional.
        crlfRequired: True if both CR and LF are required to delimit a
            line. Optional.

    Returns:
        A line of text.
    """
    print(port, timeout, encoding, crlfRequired)
    return ""


def readUntil(port, delimiter, includeDelimiter, timeout=5000):
    # type: (String, String, bool, Optional[int]) -> String
    """Reads a byte at a time from a serial port until a delimiter
    character is encountered.

    The read will block for up to timeout milliseconds before returning.

    If the delimiter is not found before the timeout period, then the
    buffer will dump, potentially resulting in data loss.

    Args:
        port: The previously configured serial port to use.
        delimiter: The delimiter to read until.
        includeDelimiter: If True, the delimiter will be included in the
            return value.
        timeout: Timeout in milliseconds. Default is 5,000. Optional.

    Returns:
        A String containing all 8-bit ASCII characters read until the
        delimiter was reached, and including the delimiter if the
        "includeDelimiter" parameter was True.
    """
    print(port, delimiter, includeDelimiter, timeout)
    return ""


def sendBreak(port, millis):
    # type: (String, int) -> None
    """Sends a break signal for approximately millis milliseconds.

    Args:
        port: The name of the serial port, e.g., "COM1" or "dev/ttyS0".
        millis: Approximate length of break signal, in milliseconds.
    """
    print(port, millis)


def write(port, toWrite, encoding="utf-8"):
    # type: (String, String, Optional[String]) -> None
    """Write a String to a serial port using the platforms default
    character encoding.

    Args:
        port: The previously configured serial port to use.
        toWrite: The String to write.
        encoding: Encoding to use when constructing the string. Defaults
            to the platform's default character set. Optional.
    """
    print(port, toWrite, encoding)


def writeBytes(port, toWrite, timeout=5000):
    # type: (String, Any, Optional[int]) -> None
    """Write a byte array to a serial port.

    Args:
        port: The previously configured serial port to use.
        toWrite: The byte array to write.
        timeout: Optional timeout in milliseconds. Default is 5,000.
            Optional.
    """
    print(port, toWrite, timeout)



from java.awt import Container
from java.lang import String


class JTextComponent(Container):
    """JTextComponent is the base class for swing text components."""

    _text = "Text"  # type: String

    def getText(self):
        # type: () -> String
        return self._text

    def setText(self, t):
        # type: (String) -> None
        self._text = t



from __future__ import print_function

__all__ = ["PyJsonObjectAdapter"]

from typing import List, Optional

from com.inductiveautomation.ignition.common.gson import JsonObject
from java.lang import Object, String
from org.python.core import PyObject


class PyJsonObjectAdapter(Object):
    def __init__(self, obj):
        # type: (JsonObject) -> None
        print(self, obj)

    def __delitem__(self, key):
        # type: (PyObject) -> None
        pass

    def __findattr_ex__(self, name):
        # type: (String) -> PyObject
        pass

    def __finditem__(self, key):
        # type: (PyObject) -> PyObject
        pass

    def __iter__(self):
        # type: () -> PyObject
        pass

    def __len__(self):
        # type: () -> int
        pass

    def __setitem__(self, key, value):
        # type: (PyObject, PyObject) -> None
        pass

    def clear(self):
        # type: () -> None
        pass

    def get(self, key, default=None):
        # type: (PyObject, Optional[PyObject]) -> PyObject
        pass

    def has_key(self, key):
        # type: (PyObject) -> bool
        pass

    def items(self):
        # type: () -> List[PyObject]
        pass

    def iteritems(self):
        # type: () -> PyObject
        pass

    def iterkeys(self):
        # type: () -> PyObject
        pass

    def itervalues(self):
        # type: () -> PyObject
        pass

    def keys(self):
        # type: () -> List[PyObject]
        pass

    def pop(self, key):
        # type: (PyObject) -> PyObject
        pass

    def popitem(self):
        # type: () -> PyObject
        pass

    def setdefault(self, key, default):
        # type: (PyObject, PyObject) -> PyObject
        pass

    def update(self, *args, **kwargs):
        # type: (PyObject, String) -> None
        pass

    def values(self):
        # type: () -> List[PyObject]
        pass



from __future__ import print_function

from copy import PyStringMap

__all__ = [
    "CodeFlag",
    "CompilerFlags",
    "PyBaseCode",
    "PyBuiltinCallable",
    "PyBuiltinMethod",
    "PyCell",
    "PyCode",
    "PyDescriptor",
    "PyException",
    "PyFrame",
    "PyMethodDescr",
    "PyNewWrapper",
    "PyObject",
    "PyStringMap",
    "PyTraceback",
    "PyType",
    "TraceFunction",
    "Visitproc",
]

from enum import Enum
from typing import Any, Iterable, Iterator, List, Optional, Tuple, Union

from java.io import PrintWriter
from java.lang import Class, Object, RuntimeException, String, StringBuilder, Throwable
from org.python.expose import TypeBuilder


class PyObject(Object):
    """All objects known to the Jython runtime system are represented by
    an instance of the class PyObject or one of its subclasses.
    """

    TYPE = None  # type: PyType

    def __init__(self, objtype=None):
        # type: (Optional[PyType]) -> None
        print(objtype)

    def __abs__(self):
        # type: () -> PyObject
        pass

    def __add__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __and__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __call__(self, *args, **kwargs):
        # type: (Any, Any) -> PyObject
        pass

    def __coerce__(self, pyo):
        # type: (PyObject) -> PyObject
        pass

    def __coerce_ex__(self, o):
        # type: (PyObject) -> Object
        pass

    def __complex__(self):
        # type: () -> complex
        pass

    def __contains__(self, o):
        # type: (PyObject) -> bool
        pass

    def __delete__(self, obj):
        # type: (PyObject) -> None
        pass

    def __delitem__(self, key):
        # type: (PyObject) -> None
        pass

    def __delslice__(self, start, stop, step=None):
        # type: (PyObject, PyObject, Optional[PyObject]) -> None
        pass

    def __dir__(self):
        # type: () -> PyObject
        pass

    def __div__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __ensure_finalizer__(self):
        # type: () -> None
        pass

    def __findattr__(self, name):
        # type: (str) -> PyObject
        pass

    def __findattr_ex__(self, name):
        # type: (str) -> PyObject
        pass

    def __finditem__(self, key):
        # type: (str) -> PyObject
        pass

    def __float__(self):
        # type: () -> float
        pass

    def __floordiv__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __get__(self, obj, type):
        # type: (PyObject, PyObject) -> PyObject
        pass

    def __getattr__(self, name):
        # type: (str) -> PyObject
        pass

    def __getitem__(self, key):
        # type: (Union[int, PyObject]) -> PyObject
        pass

    def __getnewargs__(self):
        # type: () -> Tuple[Any, ...]
        pass

    def __getslice__(self, start, stop, step=None):
        # type: (PyObject, PyObject, Optional[PyObject]) -> None
        pass

    def __hex__(self):
        # type: () -> str
        pass

    def __iadd__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __iand__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __idiv__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __idivmod__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __ifloordiv__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __ilshift__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __imod__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __imul__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __index__(self):
        # type: () -> PyObject
        pass

    def __int__(self):
        # type: () -> PyObject
        pass

    def __invert__(self):
        # type: () -> PyObject
        pass

    def __ior__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __ipow__(self, other, dummy=None):
        # type: (PyObject, Optional[PyObject]) -> PyObject
        pass

    def __irshift__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __isub__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __iter__(self):
        # type: () -> Iterator[PyObject]
        pass

    def __iternext__(self):
        pass

    def __itruediv__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __ixor__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __le__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __long__(self):
        # type: () -> PyObject
        pass

    def __lshift__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __mod__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __mul__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __neg__(self):
        # type: () -> PyObject
        pass

    def __nonzero__(self):
        # type: () -> PyObject
        pass

    def __not__(self):
        # type: () -> PyObject
        pass

    def __oct__(self):
        # type: () -> PyObject
        pass

    def __or__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __pos__(self):
        # type: () -> PyObject
        pass

    def __pow__(self, o2, o3=None):
        # type: (PyObject, Optional[PyObject]) -> PyObject
        pass

    def __radd__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __rand__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __rdiv__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __rdivmod__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __rfloordiv__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __rlshift__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __rmod__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __rmul__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __ror__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __rpow__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __rrshift__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __rshift__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __rsub__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __rtruediv__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __rxor__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __set__(self, obj, value):
        # type: (PyObject, PyObject) -> None
        pass

    def __setitem__(self, key, value):
        # type: (Union[int, PyObject, str], PyObject) -> None
        pass

    def __setslice__(self, *args):
        # type: (Any) -> None
        pass

    def __sub__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __tojava__(self, c):
        # type: (Class) -> Object
        pass

    def __truediv__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def __trunc__(self):
        # type: () -> PyObject
        pass

    def __unicode__(self):
        # type: () -> unicode
        pass

    def __xor__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def _add(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _and(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _callextra(self, *args, **kwargs):
        # type: (Any, Any) -> PyObject
        pass

    def _cmp(self, o):
        # type: (PyObject) -> int
        pass

    def _div(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _divmod(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _doget(self, *args):
        # type: (Any) -> PyObject
        pass

    def _doset(self, *args):
        # type: (Any) -> Union[bool, PyObject]
        pass

    def _eq(self, o):
        # type: (PyObject) -> PyObject
        pass

    def _floordiv(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _ge(self, o):
        # type: (PyObject) -> PyObject
        pass

    def _gt(self, o):
        # type: (PyObject) -> PyObject
        pass

    def _iadd(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _iand(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _idiv(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _idivmod(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _ifloordiv(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _ilshift(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _imod(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _imul(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _in(self, o):
        # type: (PyObject) -> PyObject
        pass

    def _ior(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _ipow(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _irshift(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _is(self, o):
        # type: (PyObject) -> PyObject
        pass

    def _isnot(self, o):
        # type: (PyObject) -> PyObject
        pass

    def _isub(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _itruediv(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _ixor(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _jcall(self, *args):
        # type: (Any) -> PyObject
        pass

    def _jcallexc(self, *args):
        # type: (Any) -> PyObject
        pass

    def _jthrow(self, t):
        # type: (Throwable) -> None
        pass

    def _le(self, o):
        # type: (PyObject) -> PyObject
        pass

    def _lshift(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _lt(self, o):
        # type: (PyObject) -> PyObject
        pass

    def _mod(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _mul(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _ne(self, o):
        # type: (PyObject) -> PyObject
        pass

    def _notin(self, o):
        # type: (PyObject) -> PyObject
        pass

    def _or(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _pow(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _rshift(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _sub(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _truediv(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def _xor(self, o2):
        # type: (PyObject) -> PyObject
        pass

    def asDouble(self):
        # type: () -> float
        pass

    def asIndex(self, err=None):
        # type: (Optional[PyObject]) -> int
        pass

    def asInt(self, index=None):
        # type: (Optional[int]) -> int
        pass

    def asIterable(self):
        # type: () -> Iterable[PyObject]
        pass

    def asLong(self, index=None):
        # type: (Optional[int]) -> long
        pass

    def asName(self, arg):
        # type: (Union[int, PyObject]) -> String
        pass

    def asString(self, index=None):
        # type: (Optional[int]) -> String
        pass

    def asStringOrNull(self, index=None):
        # type: (Optional[int]) -> String
        pass

    def bit_length(self):
        # type: () -> int
        pass

    def conjugate(self):
        # type: () -> PyObject
        pass

    def delDict(self):
        # type: () -> None
        pass

    def delType(self):
        # type: () -> None
        pass

    def dispatch__init__(self, *args, **kwargs):
        # type: (Any, Any) -> None
        pass

    def fastGetClass(self):
        # type: () -> PyObject
        pass

    def fastGetDict(self):
        # type: () -> PyObject
        pass

    def getDict(self):
        # type: () -> PyObject
        pass

    def getType(self):
        # type: () -> PyType
        pass

    def implementsDescrDelete(self):
        # type: () -> bool
        pass

    def implementsDescrGet(self):
        # type: () -> bool
        pass

    def implementsDescrSet(self):
        # type: () -> bool
        pass

    def invoke(self, *args, **kwargs):
        # type: (Any, Any) -> PyObject
        pass

    def isCallable(self):
        # type: () -> bool
        pass

    def isDataDescr(self):
        # type: () -> bool
        pass

    def isIndex(self):
        # type: () -> bool
        pass

    def isInteger(self):
        # type: () -> bool
        pass

    def isMappingType(self):
        # type: () -> bool
        pass

    def isNumberType(self):
        # type: () -> bool
        pass

    def isSequenceType(self):
        # type: () -> bool
        pass

    def noAttributeError(self, name):
        # type: (String) -> None
        pass

    @staticmethod
    def object___subclasshook__(type, subclass):
        # type: (PyType, PyObject) -> PyObject
        pass

    def readonlyAttributeError(self, name):
        # type: (String) -> None
        pass

    def setDict(self, newDict):
        # type: (PyObject) -> None
        pass

    def setType(self, type):
        # type: (PyType) -> None
        pass


class CodeFlag(Enum):
    CO_FUTURE_ABSOLUTE_IMPORT = 16384
    CO_FUTURE_DIVISION = 8192
    CO_FUTURE_PRINT_FUNCTION = 65536
    CO_FUTURE_UNICODE_LITERALS = 131072
    CO_FUTURE_WITH_STATEMENT = 32768
    CO_GENERATOR = 32
    CO_GENERATOR_ALLOWED = 4096
    CO_NESTED = 16
    CO_NEWLOCALS = 2
    CO_OPTIMIZED = 1
    CO_VARARGS = 4
    CO_VARKEYWORDS = 8
    flag = None  # type: int

    def isFlagBitSetIn(self, flags):
        # type: (int) -> bool
        pass

    @staticmethod
    def valueOf(name):
        # type: (String) -> CodeFlag
        pass

    @staticmethod
    def values():
        # type: () -> List[CodeFlag]
        pass


class CompilerFlags(Object):
    dont_imply_dedent = None  # type: bool
    encoding = None  # type: String
    only_ast = None  # type: bool
    PyCF_DONT_IMPLY_DEDENT = 512  # type: int
    PyCF_ONLY_AST = 1024  # type: int
    PyCF_SOURCE_IS_UTF8 = 256  # type: int
    source_is_utf8 = None  # type: bool

    def __init__(self, co_flags=None):
        # type: (Optional[int]) -> None
        print(co_flags)

    def combine(self, flags):
        # type: (Union[CompilerFlags, int]) -> CompilerFlags
        pass

    @staticmethod
    def getCompilerFlags(flags, frame):
        # type: (Union[CompilerFlags, int], PyFrame) -> CompilerFlags
        pass

    def isFlagSet(self, flag):
        # type: (CodeFlag) -> bool
        pass

    def toBits(self):
        # type: () -> int
        pass


class PyCode(PyObject):
    co_name = None  # type: String

    def call(self, *args):
        # type: (Any) -> None
        pass


class PyBaseCode(PyCode):
    co_argcount = None  # type: int
    co_cellvars = None  # type: List[String]
    co_filename = None  # type: String
    co_firstlineno = None  # type: int
    co_flags = None  # type: CompilerFlags
    co_freevars = None  # type: List[String]
    co_nlocals = None  # type: int
    co_varnames = None  # type: List[String]
    jy_npurecell = None  # type: int
    varargs = None  # type: bool
    varkwargs = None  # type: bool

    def getCompilerFlags(self):
        # type: () -> CompilerFlags
        pass

    def hasFreevars(self):
        # type: () -> bool
        pass


class PyBuiltinCallable(PyObject):
    def bind(self, o):
        # type: (PyObject) -> PyBuiltinCallable
        raise NotImplementedError

    def fastGetName(self):
        # type: () -> PyObject
        pass

    def getDoc(self):
        # type: () -> String
        pass

    def getModule(self):
        # type: () -> PyObject
        pass

    def getSelf(self):
        # type: () -> PyObject
        pass

    def makeCall(self):
        # type: () -> PyObject
        pass

    def setInfo(self, info):
        # type: (PyBuiltinCallable.Info) -> None
        pass

    class Info(object):
        def getMaxargs(self):
            # type: () -> int
            raise NotImplementedError

        def getMinargs(self):
            # type: () -> int
            raise NotImplementedError

        def getName(self):
            # type: () -> String
            raise NotImplementedError

        def unexpectedCall(self, nargs, keywords):
            # type: (int, bool) -> PyException
            raise NotImplementedError


class PyBuiltinMethod(PyBuiltinCallable):
    def bind(self, o):
        # type: (PyObject) -> PyBuiltinCallable
        pass

    def makeDescriptor(self, t):
        # type: (PyType) -> PyMethodDescr
        pass

    def refersDirectlyTo(self, ob):
        # type: (PyObject) -> bool
        pass

    def traverse(self, visit, arg):
        # type: (Visitproc, Object) -> int
        pass


class PyCell(PyObject):
    ob_ref = None  # type: PyObject

    def __init__(self):
        # type: () -> None
        super(PyCell, self).__init__()

    def getCellContents(self):
        # type: () -> PyObject
        pass

    def refersDirectlyTo(self, ob):
        # type: (PyObject) -> bool
        pass

    def traverse(self, visit, arg):
        # type: (Visitproc, Object) -> int
        pass


class PyDescriptor(PyObject, PyBuiltinCallable.Info):
    def __init__(self, t, func):
        # type: (PyType, PyBuiltinCallable) -> None
        super(PyDescriptor, self).__init__(t)
        print(func)

    def getDoc(self):
        # type: () -> String
        pass

    def getMaxargs(self):
        # type: () -> int
        pass

    def getMinargs(self):
        # type: () -> int
        pass

    def getName(self):
        # type: () -> String
        pass

    def getObjClass(self):
        # type: () -> PyObject
        pass

    def refersDirectlyTo(self, ob):
        # type: (PyObject) -> bool
        pass

    def traverse(self, visit, arg):
        # type: (Visitproc, Object) -> int
        pass

    def unexpectedCall(self, nargs, keywords):
        # type: (int, bool) -> PyException
        pass


class PyException(RuntimeException):
    traceback = None  # type: Optional[PyTraceback]
    type = None  # type: Optional[PyObject]
    value = None  # type: Optional[Union[PyObject, String]]

    def __init__(
        self,
        type_=None,  # type: Optional[PyObject]
        value=None,  # type: Optional[Union[PyObject, String]]
        traceback=None,  # type: Optional[PyTraceback]
    ):
        # type: (...) -> None
        super(PyException, self).__init__()
        self.type = type_
        self.value = value
        self.traceback = traceback

    @staticmethod
    def doRaise(type_, value, traceback):
        # type: (PyObject, PyObject, PyObject) -> PyException
        pass

    def exceptionClassName(self, obj):
        # type: (PyObject) -> String
        pass

    def isExceptionClass(self, obj):
        # type: (PyObject) -> bool
        pass

    def match(self, exc):
        # type: (PyObject) -> bool
        pass

    def normalize(self):
        # type: () -> None
        pass

    def refersDirectlyTo(self, ob):
        # type: (PyObject) -> bool
        pass

    def super__printStackTrace(self, w):
        # type: (PrintWriter) -> None
        pass

    def tracebackHere(self, here, isFinally=None):
        # type: (PyFrame, Optional[bool]) -> None
        pass

    def traverse(self, visit, arg):
        # type: (Visitproc, Object) -> int
        pass


class PyFrame(PyObject):
    f_back = None  # type: PyFrame
    f_builtins = None  # type: PyObject
    f_code = None  # type: PyBaseCode
    f_env = None  # type: List[PyCell]
    f_exits = None  # type: List[PyObject]
    f_fastlocals = None  # type: List[PyObject]
    f_globals = None  # type: PyObject
    f_lasti = None  # type: int
    f_lineno = None  # type: int
    f_locals = None  # type: PyObject
    f_ncells = None  # type: int
    f_nfreevars = None  # type: int
    f_savedlocals = None  # type: List[Object]
    tracefunc = None  # type: TraceFunction

    def __init__(self, *args):
        # type: (Any) -> None
        super(PyFrame, self).__init__()
        print(args)

    def checkGeneratorInput(self):
        # type: () -> Object
        pass

    def delglobal(self, index):
        # type: (String) -> None
        pass

    def dellocal(self, index):
        # type: (Union[int, String]) -> None
        pass

    def delTrace(self):
        # type: () -> None
        pass

    def getclosure(self, index):
        # type: (int) -> PyObject
        pass

    def getderef(self, index):
        # type: (int) -> PyObject
        pass

    def getf_locals(self):
        # type: () -> PyObject
        pass

    def getGeneratorInput(self):
        # type: () -> Object
        pass

    def getglobal(self, index):
        # type: (String) -> PyObject
        pass

    def getLine(self):
        # type: () -> int
        pass

    def getlocal(self, index):
        # type: (int) -> PyObject
        pass

    def getLocals(self):
        # type: () -> PyObject
        pass

    def getname(self, index):
        # type: (String) -> PyObject
        pass

    def getTrace(self):
        # type: () -> PyObject
        pass

    def refersDirectlyTo(self, ob):
        # type: (PyObject) -> bool
        pass

    def setderef(self, index, value):
        # type: (int, PyObject) -> None
        pass

    def setglobal(self, index, value):
        # type: (String, PyObject) -> None
        pass

    def setline(self, line):
        # type: (int) -> None
        pass

    def setlocal(self, index, value):
        # type: (Union[int, String], PyObject) -> None
        pass

    def setTrace(self, trace):
        # type: (PyObject) -> None
        pass

    def to_cell(self, parm_index, env_index):
        # type: (int, int) -> None
        pass

    def traverse(self, visit, arg):
        # type: (Visitproc, Object) -> int
        pass


class PyMethodDescr(PyDescriptor):
    def __init__(self, t, func):
        super(PyMethodDescr, self).__init__(t, func)


class PyNewWrapper(PyBuiltinMethod):
    for_type = None  # type: PyType

    def __init__(self, *args):
        # type: (Any) -> None
        super(PyNewWrapper, self).__init__()
        print(args)

    def getWrappedType(self):
        # type: () -> PyType
        pass

    def new_impl(self, init, subtype, *args, **kwargs):
        # type: (bool, PyType, Any, Any) -> PyObject
        raise NotImplementedError

    def setWrappedType(self, type_):
        # type: (PyType) -> None
        pass


class PyTraceback(PyObject):
    tb_frame = None  # type: PyFrame
    tb_lineno = None  # type: int
    tb_next = None  # type: PyObject

    def __init__(self, next, frame):
        # type: (PyTraceback, PyFrame) -> None
        super(PyTraceback, self).__init__()
        print(next, frame)

    def dumpStack(self, buf=None):
        # type: (Optional[StringBuilder]) -> Optional[String]
        pass

    def refersDirectlyTo(self, ob):
        # type: (PyObject) -> bool
        pass

    def traverse(self, visit, arg):
        # type: (Visitproc, Object) -> int
        pass


class PyType(PyObject):
    @staticmethod
    def addBuilder(c, builder):
        # type: (Class, TypeBuilder) -> None
        pass

    def addMethod(self, meth):
        # type: (PyBuiltinMethod) -> None
        pass

    def compatibleForAssignment(self, other, attribute):
        # type: (PyType, String) -> None
        pass

    def delBases(self):
        # type: () -> None
        pass

    def delModule(self):
        # type: () -> None
        pass

    @staticmethod
    def ensureBootstrapped():
        # type: () -> bool
        pass

    @staticmethod
    def ensureDoc(arg):
        # type: (PyObject) -> None
        pass

    @staticmethod
    def ensureModule(arg):
        # type: (PyObject) -> None
        pass

    def fastGetName(self):
        # type: () -> String
        pass

    @staticmethod
    def fromClass(c, hardRef=False):
        # type: (Class, bool) -> PyType
        pass

    def getAbstractMethods(self):
        # type: () -> PyObject
        pass

    def getBase(self):
        # type: () -> PyObject
        pass

    def getBases(self):
        # type: () -> PyObject
        pass

    def getDoc(self):
        # type: () -> PyObject
        pass

    def getFlags(self):
        # type: () -> long
        pass

    def getModule(self):
        # type: () -> PyObject
        pass

    def getMro(self):
        # type: () -> Tuple[Any, ...]
        pass

    def getName(self):
        # type: () -> String
        pass

    def getNumSlots(self):
        # type: () -> int
        pass

    def getProxyType(self):
        # type: () -> Class
        pass

    def getStatic(self):
        # type: () -> PyObject
        pass

    def instDict(self):
        # type: () -> PyObject
        pass

    def isSubType(self, supertype):
        # type: (PyType) -> bool
        pass

    def lookup_where(self, name, where):
        # type: (String, List[PyObject]) -> PyObject
        pass

    def lookup(self, name):
        # type: (String) -> PyObject
        pass

    def needsFinalizer(self):
        # type: () -> bool
        pass

    @staticmethod
    def newType(
        new_,  # type: PyNewWrapper
        metatype,  # type: PyType
        name,  # type: String
        bases,  # type: Tuple[Any, ...]
        dict_,  # type: PyObject
    ):
        # type: (...) -> PyObject
        pass

    def pyDelName(self):
        # type: () -> None
        pass

    def pyGetName(self):
        # type: () -> PyObject
        pass

    def pySetName(self, name):
        # type: (PyObject) -> None
        pass

    def refersDirectlyTo(self, ob):
        # type: (PyObject) -> bool
        pass

    def removeMethod(self, meth):
        # type: (PyBuiltinMethod) -> None
        pass

    def setAbstractMethods(self, value):
        # type: (PyObject) -> None
        pass

    def setBases(self, newBasesTuple):
        # type: (PyObject) -> None
        pass

    def setName(self, name):
        # type: (String) -> None
        pass

    def super_lookup(self, ref, name):
        # type: (PyType, String) -> PyObject
        pass

    def traverse(self, visit, arg):
        # type: (Visitproc, Object) -> int
        pass

    def type___eq__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def type___ge__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def type___gt__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def type___instancecheck__(self, inst):
        # type: (PyObject) -> bool
        pass

    def type___le__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def type___lt__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def type___ne__(self, other):
        # type: (PyObject) -> PyObject
        pass

    def type___subclasscheck__(self, inst):
        # type: (PyObject) -> bool
        pass

    def type___subclasses__(self):
        # type: () -> PyObject
        pass


class TraceFunction(Object):
    def traceCall(self, frame):
        # type: (PyFrame) -> TraceFunction
        raise NotImplementedError

    def traceException(self, frame, exc):
        # type: (PyFrame, PyException) -> TraceFunction
        raise NotImplementedError

    def traceLine(self, frame, line):
        # type: (PyFrame, int) -> TraceFunction
        raise NotImplementedError

    def traceReturn(self, frame, ret):
        # type: (PyFrame, PyObject) -> TraceFunction
        raise NotImplementedError


class Visitproc(object):
    def visit(self, obj, arg):
        # type: (PyObject, Object) -> int
        pass



from __future__ import print_function

__all__ = ["BrowseTag", "OPCBrowseTag"]

from typing import Optional

from com.inductiveautomation.ignition.common.opc import BrowseElementType
from com.inductiveautomation.ignition.common.sqltags.model.types import DataType
from com.inductiveautomation.ignition.common.tags.config.types import TagObjectType
from java.lang import Class, Object


class BrowseTag(Object):
    dataType = None  # type: DataType
    name = None  # type: str
    fullPath = None  # type: str
    path = None  # type: str
    type = None  # type: TagObjectType
    valueSource = None  # type: str

    def __init__(
        self,
        name,  # type: str
        path,  # type: str
        fullPath,  # type: str
        type_,  # type: TagObjectType
        valueSource,  # type: str
        dataType,  # type: DataType
    ):
        self.name = name
        self.path = path
        self.fullPath = fullPath
        self.type = type_
        self.valueSource = valueSource
        self.dataType = dataType

    def getDataType(self):
        return self.dataType

    def getFullPath(self):
        return self.fullPath

    def getPath(self):
        return self.path

    def getTagType(self):
        return self.type

    def getValueSource(self):
        return self.valueSource

    def isDB(self):
        print(self)
        return True

    def isExpression(self):
        print(self)
        return True

    def isFolder(self):
        print(self)
        return True

    def isMemory(self):
        print(self)
        return True

    def isOPC(self):
        print(self)
        return True

    def isQuery(self):
        print(self)
        return True

    def isUDT(self):
        print(self)
        return True


class OPCBrowseTag(Object):
    def __init__(
        self,
        opcServer=None,  # type: Optional[str]
        type_=None,  # type: Optional[BrowseElementType]
        displayName=None,  # type: Optional[str]
        displayPath=None,  # type: Optional[str]
        dataType=None,  # type: Optional[Class]
        opcItemPath=None,  # type: Optional[str]
    ):
        # type: (...) -> None
        self._opcServer = opcServer
        self._type = type_
        self._displayName = displayName
        self._displayPath = displayPath
        self._dataType = dataType
        self._opcItemPath = opcItemPath

    def getDataType(self):
        return self._dataType

    def getDisplayName(self):
        return self._displayName

    def getDisplayPath(self):
        return self._displayPath

    def getOpcItemPath(self):
        return self._opcItemPath

    def getOpcServer(self):
        return self._opcServer

    def getType(self):
        return self._type



"""Navigation Functions.

The following functions allow you to open and close windows in the
client.
"""

from __future__ import print_function

__all__ = [
    "centerWindow",
    "closeParentWindow",
    "closeWindow",
    "desktop",
    "getCurrentWindow",
    "goBack",
    "goForward",
    "goHome",
    "openWindow",
    "openWindowInstance",
    "swapTo",
    "swapWindow",
]

from typing import Any, Dict, Optional, Union

from com.inductiveautomation.factorypmi.application import FPMIWindow
from com.inductiveautomation.factorypmi.application.script.builtin import NavUtilities
from java.lang import String
from java.util import EventObject


def centerWindow(arg):
    # type: (Union[String, FPMIWindow]) -> None
    """Given a window path, or a reference to a window itself, it will
    center the window.

    The window should be floating and non-maximized. If the window can't
    be found, this function will do nothing.

    Args:
        arg: The path of the window or a reference to the window to
            center.
    """
    print(arg)


def closeParentWindow(event):
    # type: (EventObject) -> None
    """Closes the parent window given a component event object.

    Args:
        event: A component event object. The enclosing window for the
            component will be closed.
    """
    print(event)


def closeWindow(arg):
    # type: (Union[String, FPMIWindow]) -> None
    """Given a window path, or a reference to a window itself, it will
    close the window.

    If the window can't be found, this function will do nothing.

    Args:
        arg: The path of the window or a reference to the window to
            center.
    """
    print(arg)


def desktop(handle="primary"):
    # type: (Optional[String]) -> NavUtilities
    """Allows for invoking system.nav functions on a specific desktop.

    Args:
        handle: The handle for the desktop to use. May be omitted for
            the primary desktop. Optional.

    Returns:
        A copy of system.nav that will alter the desktop named by the
        given handle.
    """
    print(handle)
    return NavUtilities()


def getCurrentWindow():
    # type: () -> String
    """Returns the path of the current "main screen" window, which is
    defined as the maximized window.

    With the typical navigation, there is only ever one maximized window
    at a time.

    Returns:
        The path of the current "main screen" window - the maximized
        window.
    """
    return "Path/To/Maximized Window"


def goBack():
    # type: () -> FPMIWindow
    """When using the typical navigation strategy, this function will
    navigate back to the previous main screen window.

    Returns:
        A reference to window that was navigated to.
    """
    return FPMIWindow("Back")


def goForward():
    # type: () -> FPMIWindow
    """When using the typical navigation strategy, this function will
    navigate "forward" to the last main screen window the user was on
    when they executed a system.nav.goBack().

    Returns:
        A reference to window that was navigated to.
    """
    return FPMIWindow("Forward")


def goHome():
    # type: () -> FPMIWindow
    """When using the typical navigation strategy, this function will
    navigate to the "home" window.

    This is automatically detected as the first main-screen window shown
    in a project.

    Returns:
        A reference to window that was navigated to.
    """
    return FPMIWindow("Home")


def openWindow(path, params=None):
    # type: (String, Optional[Dict[String, Any]]) -> FPMIWindow
    """Opens the window with the given path.

    If the window is already open, brings it to the front. The optional
    params dictionary contains key:value pairs which will be used to set
    the target window's root container's dynamic variables.

    Args:
        path: The path to the window to open.
        params: A dictionary of parameters to pass into the window. The
            keys in the dictionary must match dynamic property names on
            the target window's root container. The values for each key
            will be used to set those properties. Optional.

    Returns:
        A reference to the opened window.
    """
    print(path, params)
    return FPMIWindow("Opened Window")


def openWindowInstance(path, params=None):
    # type: (String, Optional[Dict[String, Any]]) -> FPMIWindow
    """Operates exactly like system.nav.openWindow, except that if the
    named window is already open, then an additional instance of the
    window will be opened.

    There is no limit to the number of additional instances of a window
    that you can open.

    Args:
        path: The path to the window to open.
        params: A dictionary of parameters to pass into the window. The
            keys in the dictionary must match dynamic property names on
            the target window's root container. The values for each key
            will be used to set those properties. Optional.

    Returns:
        A reference to the opened window.
    """
    print(path, params)
    return FPMIWindow("Window Instance")


def swapTo(path, params=None):
    # type: (String, Optional[Dict[String, Any]]) -> FPMIWindow
    """Performs a window swap from the current main screen window to the
    window specified.

    Swapping means that the opened window will take the place of the
    closing window - in this case it will be maximized.

    This function works like system.nav.swapWindow except that you
    cannot specify the source for the swap.

    Args:
        path: The path to the window to open.
        params: A dictionary of parameters to pass into the window. The
            keys in the dictionary must match dynamic property names on
            the target window's root container. The values for each key
            will be used to set those properties. Optional.

    Returns:
        A reference to the swapped-to window.
    """
    print(path, params)
    return FPMIWindow("Swapped To")


def swapWindow(
    arg,  # type: Union[String, EventObject]
    swapToPath,  # type: String
    params=None,  # type: Optional[Dict[String, Any]]
):
    # type: (...) -> FPMIWindow
    """Performs a window swap.

    This means that one window is closed, and another is opened and
    takes its place - assuming its size, floating state, and
    maximization state. This gives a seamless transition; one window
    seems to simply turn into another.

    This function works like system.nav.swapTo except that you can
    specify the source and destination for the swap.

    Args:
        arg: The path of the window to swap from. Must be a currently
            open window, otherwise this will act like an openWindow, or
            a component event (EventObject) whose enclosing window will
            be used as the "swap-from" window.
        swapToPath: The name of the window to swap to.
        params: A dictionary of parameters to pass into the window. The
            keys in the dictionary must match dynamic property names on
            the target window's root container. The values for each key
            will be used to set those properties. Optional.

    Returns:
        A reference to the swapped-to window.
    """
    print(arg, swapToPath, params)
    return FPMIWindow("Swapped To")
